//https://www.lawebdelprogramador.com/foros/Java/1675047-programa-para-ver-si-las-campanas-del-instituto-son-viables-o-no.html
package bombones;

import java.util.Scanner;

public final class Test {
	
	/**
	 * Método recursivo que comprueba si la campaña de bombones regalo es viable.<br>
	 * En cada iteracion del método, este calcula los bombones extra que obtiene el alumno
	 * según las condiciones de la campaña.<br>Si los bombones extra calculados son inferiores
	 * al numero de envoltorios necesarios para obtener un regalo, la campaña se considera <b>VIABLE</b>
	 * y se informa por pantalla del total de bombones que ha conseguido comer el alumno y cuantos envoltorios
	 * no canjeables le han sobrado.<br>En cambio, si los bombones extra calculados igualan o superan al número de
	 * envoltorios necesarios para canjear, significa que aún no sabemos si la campaña es viable y entonces se hará
	 * una llamada recursiva a este método pasándole los parámetros actualizados tras los cálculos realizados.<br>
	 * Este método lleva la cuenta de cuantas llamadas recursivas se ha realizado, en caso de llegar a las 1000 llamadas
	 * se considera que la campaña <b>NO ES VIABLE</b> ya que el alumno habría obtenido miles de bombones gratuitos y 
	 * parece que aún seguiría obteniendo muchos más.<br>
	 * <b><u>--MUY IMPORTANTE--</u></b> A pesar de llevar un control de iteraciones, es muy probable que ocurra una 
	 * excepción por <i>desbordamiento de pila</i>(StackOverflowError) según las condiciones de la campaña ya que se pueden
	 * acumular más bombones de los que son posibles contar.<br>Debido a esto se hace necesario controlar esta excepcion
	 * cuando se llame a este método.<br>Obviamente, si ocurre esta excepción, habrá que considerar la campaña <b>NO VIABLE</b>
	 * @param envol Número de envoltorios necesarios para obtener más bombones.
	 * @param bombRegalo Número de bombones que se regalan en cada canje de envoltorios.
	 * @param bombComprados Este parámetro representará dos valores distintos:<br>
	 * <ul><li>La primera vez que llamamos al método, representa los bombones que ha comprado el alumno</li>
	 * <li>Las siguientes veces que este metodo sea llamado recursivamente, este parámetro representa los "bombones
	 * extra" que se han conseguido con el último canje de envoltorios.</li></ul>
	 * @param bombComidos Número de bombones totales que ha conseguido comer el alumno gracias a la campaña
	 * @param iteraciones Número de veces que este método se ha llamado así mismo recursivamente.
	 * @return <b>true</b> si la campaña se considera viable, <b>false</b> en caso contrario.
	 */
	static boolean esViable(int envol, int bombRegalo, int bombComprados, int bombComidos, int iteraciones) {
		
		bombComidos += bombComprados;//La primera iteracion, el alumno come los que ha comprado.
		                             //En siguientes iteraciones, se irán sumando los bombones extra que pueda conseguir
		
		if (iteraciones == 1000)
			return false;//Tras mil llamada recursivas, podemos deducir que la campaña es un fracaso...
		else
		{
			int bombonesExtra = bombComprados / envol * bombRegalo;//Calculamos los bombones extra que consigue según condiciones
			
			if (bombonesExtra < envol)//En algun momento el alumno obtiene bombones insuficientes para seguir teniendo premio.
			{
				System.out.printf("\n\nEl alumno comería %d bombones en total.", (bombComidos+bombonesExtra));
				System.out.println("\nEnvoltorios sobrantes no canjeables: " + bombonesExtra);
				return true; //CAMPAÑA VIABLE
			}
			else //De momento, el alumno sigue obteniendo bombones suficientes para seguir canjeando..
				return esViable(envol, bombRegalo, bombonesExtra, bombComidos, iteraciones++);
		} 
	}

	public static void main(String[] args) {
		Scanner teclado = new Scanner(System.in);
		int envoltorios = 0;
		int bombonesRegalo = 0;
		int bombonesComprados = 0;
		char continuar = 's';
		
		while(continuar == 's')
		{
			System.out.print("\nNumero de envoltorios necesarios para regalar mas bombones: ");
			envoltorios = Integer.parseInt(teclado.nextLine());
			System.out.print("Numero de bombones a regalar: ");
			bombonesRegalo = Integer.parseInt(teclado.nextLine());
			System.out.print("Numero de bombones comprados: ");
			bombonesComprados = Integer.parseInt(teclado.nextLine());
			
			/*
			 * El metodo recursivo esViable() es muy susceptible de provocar "desbordamiento de pila" si ponemos
			 * unas condiciones de campaña demasiado "generosas", ya que las continuas llamadas recursivas
			 * acumularán más bombones de los que el sistema puede contar.
			 * Se puede comprobar con los valores:
			 * Numero de envoltorios necesarios para regalar mas bombones: 1
			 * Numero de bombones a regalar: 1
			 * Numero de bombones comprados: 10
			 * Para controlar esta excepcion usamos try catch.
			 * En caso de ocurrir dicha excepcion, la campaña ha de ser considera NO VIABLE, pues semejante número de
			 * bombones de regalo serían la ruina del colegio, y probablemente la muerte del alumno por indigestion.
			 */
			try {
				System.out.println(esViable(envoltorios, bombonesRegalo, bombonesComprados, 0, 0)?"\nCAMPAÑA VIABLE":"\nCAMPAÑA NO VIABLE");
			} catch (StackOverflowError e) {
				System.out.println("\nCAMPAÑA NO VIABLE");
			}
			
			System.out.print("\n¿Desea continuar(s/n): ");
			continuar = teclado.nextLine().toLowerCase().charAt(0);
		}
		System.out.println("\n\tFIN DE PROGRAMA");
		teclado.close();
	}

}
